@@ -1,4 +1,5 @@
 use crate::buffer::{EventBuffer, Get, MixedEventBuffer, State};
+use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex};
 use std::thread::{self, JoinHandle};
 use std::time::Duration;
@@ -6,25 +7,41 @@ use std::time::Duration;
 struct DebouncerThread<B> {
     mutex: Arc<Mutex<B>>,
     thread: JoinHandle<()>,
+    stopped: Arc<AtomicBool>,
 }
 
 impl<B> DebouncerThread<B> {
-    fn new<F>(buffer: B, f: F) -> Self
+    fn new<F>(buffer: B, mut f: F) -> Self
     where
         B: Get + Send + 'static,
-        F: Fn(B::Data) -> () + Send + 'static,
+        F: FnMut(B::Data) -> () + Send + 'static,
     {
         let mutex = Arc::new(Mutex::new(buffer));
-        let mutex_clone = mutex.clone();
-        let thread = thread::spawn(move || loop {
-            let state = mutex_clone.lock().unwrap().get();
-            match state {
-                State::Empty => thread::park(),
-                State::Wait(duration) => thread::sleep(duration),
-                State::Ready(data) => f(data),
+        let stopped = Arc::new(AtomicBool::new(false));
+        let thread = thread::spawn({
+            let mutex = mutex.clone();
+            let stopped = stopped.clone();
+            move || {
+                while !stopped.load(Ordering::Relaxed) {
+                    let state = mutex.lock().unwrap().get();
+                    match state {
+                        State::Empty => thread::park(),
+                        State::Wait(duration) => thread::sleep(duration),
+                        State::Ready(data) => f(data),
+                    }
+                }
             }
         });
-        Self { mutex, thread }
+        Self {
+            mutex,
+            thread,
+            stopped,
+        }
+    }
+
+    fn stop(self) -> JoinHandle<()> {
+        self.stopped.store(true, Ordering::Relaxed);
+        self.thread
     }
 }
 
@@ -36,7 +53,7 @@ pub struct EventDebouncer<T>(DebouncerThread<EventBuffer<T>>);
 impl<T: PartialEq> EventDebouncer<T> {
     pub fn new<F>(delay: Duration, f: F) -> Self
     where
-        F: Fn(T) -> () + Send + 'static,
+        F: FnMut(T) -> () + Send + 'static,
         T: Send + 'static,
     {
         Self(DebouncerThread::new(EventBuffer::new(delay), f))
@@ -46,6 +63,15 @@ impl<T: PartialEq> EventDebouncer<T> {
         self.0.mutex.lock().unwrap().put(data);
         self.0.thread.thread().unpark();
     }
+
+    /// Signals the debouncer thread to quit and returns a
+    /// [std::thread::JoinHandle] which can be `.join()`ed in the consumer
+    /// thread. The common idiom is:
+    ///
+    ///     debouncer.stop().join().unwrap();
+    pub fn stop(self) -> JoinHandle<()> {
+        self.0.stop()
+    }
 }
 
 /// Threaded debouncer wrapping [MixedEventBuffer]. Accepts a callback function
@@ -57,7 +83,7 @@ pub struct MixedEventDebouncer<T>(DebouncerThread<MixedEventBuffer<T>>);
 impl<T: Eq> MixedEventDebouncer<T> {
     pub fn new<F>(f: F) -> Self
     where
-        F: Fn(T) -> () + Send + 'static,
+        F: FnMut(T) -> () + Send + 'static,
         T: Send + 'static,
     {
         Self(DebouncerThread::new(MixedEventBuffer::new(), f))
@@ -67,6 +93,15 @@ impl<T: Eq> MixedEventDebouncer<T> {
         self.0.mutex.lock().unwrap().put(data, delay);
         self.0.thread.thread().unpark();
     }
+
+    /// Signals the debouncer thread to quit and returns a
+    /// [std::thread::JoinHandle] which can be `.join()`ed in the consumer
+    /// thread. The common idiom is:
+    ///
+    ///     debouncer.stop().join().unwrap();
+    pub fn stop(self) -> JoinHandle<()> {
+        self.0.stop()
+    }
 }
 
 #[cfg(test)]
